/* 
 * nre.h --
 *
 *	Definitions for the nre regular expression package.
 *
 * Copyright (c) 1997 Darrel Schneider
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef _REGEXP
#define _REGEXP 1

#ifndef _TCL
#include "tcl.h"
#endif

enum RegExpFlags {
  REGEXP_NOCASE = 1,   /* (?i) make the regex case insensitive */
  REGEXP_PERLMODE = 2, /* (?p) recognize perl shortcuts in particular:
			*      \<, \>, \b, \B, \d, \D, \s, \S, \w, \W, \Q,
                        *      \A, \Z, \G
			*      {x,y}
			*/
  REGEXP_MLINEMODE = 4,/* (?m) ^ and $ match the beginning and end of lines
			*      in addition to the begin and end of the string.
			*      Also . matches \n.
			*/
  REGEXP_XMODE = 8,    /* (?x) turns on extended formatting of regexps.
			*      This allows you to embed whitespace
			*      and comments in regular expressions
			*      to make them more readable.
			*/
};

typedef struct regexp {
        int regexpSize;         /* True size of the regexp block of memory */
	char* regstartnode;	/* Internal use only. */
	char *regmust;		/* Internal use only. */
	char *regclasses;	/* Internal use only. */
	int regmlen;		/* Internal use only. */
        int regflags;           /* Bit mask of RegExpFlags */
        int subexpCount;        /* number of sub expressions */
	char program[1];	/* Internal use only. */
} regexp;


/* Following could be in tcl.h */
/*
 * NEWNSUBEXP must be at least 10, and no greater than 256 or the parser
 * will not work properly.
 */

#define NEWNSUBEXP  256

/* Definitions for regexp result */
typedef struct Tcl_RegExpExecResult {
  char* startp[NEWNSUBEXP];
  char* endp[NEWNSUBEXP];
} Tcl_RegExpExecResult;

/* UCHAR taken from tclInt.h */
/*
 * The macro below is used to modify a "char" value (e.g. by casting
 * it to an unsigned character) so that it can be used safely with
 * macros such as isspace.
 */
#ifndef UCHAR
#define UCHAR(c) ((unsigned char) (c))
#endif

EXTERN regexp *TclNewRegComp _ANSI_ARGS_((char *exp));
EXTERN regexp *TclNewRegCompSz _ANSI_ARGS_((char *exp, int length, int noCase));
EXTERN int TclNewRegExec _ANSI_ARGS_((regexp *prog, char *string, char *start,
				   Tcl_RegExpExecResult *resptr));
EXTERN int TclNewRegExecSz _ANSI_ARGS_((regexp *prog, char *string, char *start,
				     int length, Tcl_RegExpExecResult *resptr));
EXTERN void TclNewRegError _ANSI_ARGS_((char *msg));
EXTERN char *TclNewGetRegError _ANSI_ARGS_((void));
EXTERN int TclNewRegDump _ANSI_ARGS_((Tcl_Interp *interp, regexp *r));

#if 0
/* Following could be in tcl.h */
EXTERN Tcl_Obj*	Tcl_RegExpCompileObj _ANSI_ARGS_((Tcl_Interp *interp,
						  Tcl_Obj *pattern, int noCase));
EXTERN int		Tcl_RegExpExecObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tcl_Obj *reObj, char *string, char *start,
			    int stringLength,
			    Tcl_RegExpExecResult *resptr));
EXTERN int		Tcl_RegExpMatchObj _ANSI_ARGS_((Tcl_Interp *interp,
							char *string,
							int stringLength,
							Tcl_Obj *patternObj));
EXTERN void		Tcl_RegExpRangeObj _ANSI_ARGS_((Tcl_Obj* reObj,
			    Tcl_RegExpExecResult *resptr,
			    int index, char **startPtr, char **endPtr));
#endif

#endif /* REGEXP */
